#include "DHT.h"
#include "max6675.h"
#include<mwc_stepper.h>
#include <ThreeWire.h>
#include <RtcDS1302.h>
#define DHTPIN 2     // Digital pin connected to the DHT sensor
#define DHTPIN2 31     // Digital pin connected to the DHT sensor
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321

//global setup for timer
ThreeWire myWire(34, 35, 36); // IO, SCLK, CE
RtcDS1302<ThreeWire> Rtc(myWire);
RtcDateTime start_time;
unsigned int delay_h = 0, delay_m = 0, delay_s = 0,
             target_d = 0, target_h = 0, target_m = 0, target_s = 0;
bool time_elapsed = false;

const int globalCtrl = 40; //triggered when non compostable detected or empty
const int startButton = 9;
const int heaterRelay = 23;
const int intakeFan = 13;
const int exaustFan = 12;

const int blenderMotor = 11;
const int blenderDir = 27;
const int blenderEn = 52;

const int compressorMotor = 10;
const int compressorDir = 26;
const int compressorEn = 51;

const int thermoDO = 7;
const int thermoCS = 6;
const int thermoCLK = 5;
const int juicerStop = 4;
const int juicerFast = 3;
const int contact = 42;

MAX6675 thermocouple(thermoCLK, thermoCS, thermoDO);
MWCSTEPPER blender(blenderEn, blenderDir, blenderMotor);
MWCSTEPPER compressor(compressorEn, compressorDir, compressorMotor);
DHT dht(DHTPIN, DHTTYPE);
DHT dht2(DHTPIN2, DHTTYPE);
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  Serial.println("Composter Begin");
  dht.begin();
  dht2.begin();
  blender.init();
  compressor.init();
  pinMode(globalCtrl, INPUT);
  pinMode(heaterRelay, OUTPUT);
  pinMode(intakeFan, OUTPUT);
  pinMode(exaustFan, OUTPUT);
  pinMode(juicerStop, OUTPUT);
  pinMode(juicerFast, OUTPUT);
  pinMode(blenderMotor, OUTPUT);

  // set up timer
  Rtc.Begin();

  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);

  if (!Rtc.IsDateTimeValid())
  {
    // Common Causes:
    //    1) first time you ran and the device wasn't running yet
    //    2) the battery on the device is low or even missing

    Rtc.SetDateTime(compiled);
  }

  if (Rtc.GetIsWriteProtected())
  {
    Rtc.SetIsWriteProtected(false);
  }

  if (!Rtc.GetIsRunning())
  {
    Rtc.SetIsRunning(true);
  }

  RtcDateTime now = Rtc.GetDateTime();
  if (now < compiled)
  {
    Rtc.SetDateTime(compiled);
  }
  resetDelay(6, 0, 0); // 00:00:00 in hh:mm:ss


  delay(1000);

  while(digitalRead(globalCtrl) == LOW){
    Serial.println("No Compostable detected");
  }

  if (digitalRead(startButton) == HIGH) {// && !time_elapsed
    Serial.println("Composter Initiated");
    // Intaking
    digitalWrite(juicerStop, HIGH);
    delay(300);
    digitalWrite(juicerStop, LOW);
    delay(300);
    digitalWrite(juicerFast, HIGH);
    delay(300);
    digitalWrite(juicerFast, LOW);
    Serial.println("Juicer Activated");
    compressor.active(LOW);
    Serial.println("Compresspr Activated");
    bool direction = 0;
    for (size_t j = 0; j < 10; j++){
      direction = !direction;
      compressor.set(direction, 200, 200);
      for (size_t i = 0; i < 3000; i++)
      {
        compressor.run();
      }
      delay(500);
      if(digitalRead(globalCtrl) == LOW){
        while(true){
          Serial.println("No Compostable Detected, Please Restart");
          delay(1000);
        }
      }
    }
    compressor.active(HIGH);
    Serial.println("Compresspr Complete");
    digitalWrite(juicerStop, HIGH);
    delay(500);
    digitalWrite(juicerStop, LOW);
    Serial.println("Juicer Complete");
  }
  // composting
  Serial.println("Composting Start");
}

void loop() {

  while ( digitalRead(startButton) == HIGH && digitalRead(contact) == HIGH && !time_elapsed) { // digitalRead(globalCtrl) == HIGH && //digitalRead(contact) == HIGH && !time_elapsed
    //update timer and check if time elapsed
    RtcDateTime now = Rtc.GetDateTime();
    checkIsTimeElapsed(now);

    float heaterTemp = thermocouple.readCelsius();
    delay(50);
    float chamberTemp = dht.readTemperature();
    float chamberHum = dht.readHumidity();
    float electTemp = dht2.readTemperature();
    Serial.print("Pad Temp: ");
    Serial.print(heaterTemp);
    Serial.print(" Chamber Temp: ");
    Serial.print(chamberTemp);
    Serial.print(" Chamber Humidity: ");
    Serial.print(chamberHum);
    Serial.print(" Electronics Temp: ");
    Serial.println(electTemp);
    // Blender Motor
    blender.set(1, 120, 1600);
    Serial.println("blender Activated");

    for (size_t i = 0; i < 1600; i++)
    {
      blender.run();
    }
    // temp control
    if (heaterTemp <= 55) {
      digitalWrite(heaterRelay, HIGH);
      Serial.println("Heater Active");
    }
    else {
      digitalWrite(heaterRelay, LOW);
      Serial.println("Heater Deactivated");
    }
    // humidity control
    if (chamberHum >= 90) {
      analogWrite(intakeFan, 255);
      analogWrite(exaustFan, 128);
    }
    else if (chamberHum >= 70) {
      analogWrite(intakeFan, 128);
      analogWrite(exaustFan, 64);
    }
    else {
      analogWrite(intakeFan, 0);
      analogWrite(exaustFan, 64);
    }
  }
  digitalWrite(heaterRelay, LOW);
  Serial.println("Exiting");
}

void updateTargetTime() {
  unsigned int s_hour = start_time.Hour();
  unsigned int s_min = start_time.Minute();
  unsigned int s_sec = start_time.Second();
  unsigned int s_day = start_time.Day();
  unsigned int m_cout = 0;
  unsigned int h_cout = 0;
  unsigned int d_cout = 0;
  target_s = s_sec + delay_s;
  if (target_s >= 60) {
    target_s -= 60;
    m_cout = 1;
  }
  target_m = s_min + delay_m + m_cout;
  if (target_m >= 60) {
    target_m -= 60;
    h_cout = 1;
  }
  target_h = s_hour + delay_h + h_cout;
  if (target_h >= 24) {
    target_h -= 24;
    d_cout = 1;
  }
  target_d = s_day + d_cout; //ignore day round up
}

void resetDelay(unsigned int new_delay_h, unsigned int new_delay_m, unsigned int new_delay_s) {
  time_elapsed = false;
  start_time = Rtc.GetDateTime();
  delay_h = new_delay_h;
  delay_m = new_delay_m;
  delay_s = new_delay_s;
  updateTargetTime();
}

void checkIsTimeElapsed(const RtcDateTime& now) {
  unsigned int day = now.Day();
  unsigned int hour = now.Hour();
  unsigned int minute = now.Minute();
  unsigned int sec = now.Second();
  if (day == target_d && hour == target_h && minute == target_m) { //precision to min
    time_elapsed = true;
  }
}
